@@ -52,7 +52,7 @@
  # information on the Apache Software Foundation, please see
  # <http://www.apache.org/>.
  #
- # $Id: tests.py 265736 2005-09-01 14:05:43Z nlehuen $
+ # $Id: tests.py 332832 2005-11-12 20:11:59Z nlehuen $
  #
 
 # mod_python tests
@@ -383,7 +383,7 @@ class SimpleTestCase(unittest.TestCase):
 
         log("req.get_options(): %s" % `req.get_options()`)
         if req.get_options() != apache.table({"testing":"123"}):
-            self.fail("get_options() should contain 'testing':'123'")
+            self.fail("get_options() should contain 'testing':'123', contains %s"%req.get_options().items())
 
     def test_req_get_remote_host(self):
 
@@ -685,6 +685,46 @@ def req_sendfile(req):
     # os.remove(fname)
     return apache.OK
 
+def req_sendfile2(req):
+
+    import tempfile
+    fname  = tempfile.mktemp("txt")
+    f = open(fname, "w")
+    f.write("0123456789"*100);
+    f.close()
+
+    req.sendfile(fname)
+
+    # os.remove(fname)
+    return apache.OK
+ 
+def req_sendfile3(req):
+    """Check if sendfile handles symlinks properly.
+       This is only valid on posix systems.
+    """
+
+    import tempfile
+    # note mktemp is deprecated in python 2.3. Should use mkstemp instead.
+    fname  = tempfile.mktemp("txt")
+    f = open(fname, "w")
+    f.write("0123456789"*100);
+    f.close()
+    fname_symlink =  '%s.lnk' % fname
+    os.symlink(fname, fname_symlink)
+    req.sendfile(fname_symlink)
+    os.remove(fname_symlink)
+    os.remove(fname)
+    return apache.OK
+
+def fileupload(req):
+    from mod_python import util
+    import md5
+    fields = util.FieldStorage(req)
+    f = fields.getfirst('testfile')
+    
+    req.write(md5.new(f.file.read()).hexdigest())
+    return apache.OK
+
 def srv_register_cleanup(req):
 
     req.cleanup_data = "test ok"
@@ -701,6 +741,13 @@ def apache_register_cleanup(req):
 
     return apache.OK
 
+def apache_exists_config_define(req):
+    if apache.exists_config_define('FOOBAR'):
+        req.write('FOOBAR')
+    else:
+        req.write('NO_FOOBAR')
+    return apache.OK
+
 def util_fieldstorage(req):
 
     from mod_python import util
@@ -825,6 +872,11 @@ def Session_Session(req):
         req.write("test ok")
 
     return apache.OK
+
+def test_sys_argv(req):
+    import sys
+    req.write(repr(sys.argv))
+    return apache.OK
         
 def PythonOption_items(req):
     options = req.get_options().items()
