@@ -13,7 +13,7 @@
  # implied.  See the License for the specific language governing
  # permissions and limitations under the License.
  #
- # $Id: httpdconf.py 102650 2004-02-16 19:49:15Z grisha $
+ # $Id: httpdconf.py 280650 2005-09-13 20:35:57Z nlehuen $
  #
  # Config maker, a la HTMLGen. This could grow into something useful.
  #
@@ -130,7 +130,11 @@ class LogFormat(Directive):
 
 class LockFile(Directive):
     def __init__(self, val):
-        Directive.__init__(self, self.__class__.__name__, val)
+        import sys
+        if sys.platform!='win32':
+            Directive.__init__(self, self.__class__.__name__, val)
+        else:
+            Directive.__init__(self, '#'+self.__class__.__name__, val)
 
 class MaxClients(Directive):
     def __init__(self, val):
@@ -252,6 +256,14 @@ class TypesConfig(Directive):
     def __init__(self, val):
         Directive.__init__(self, self.__class__.__name__, val)
 
+class PythonInterpPerDirectory(Directive):
+    def __init__(self, val='Off'):
+        Directive.__init__(self, self.__class__.__name__, val)
+
+class PythonInterpPerDirective(Directive):
+    def __init__(self, val='Off'):
+        Directive.__init__(self, self.__class__.__name__, val)
+
 class VirtualHost(ContainerTag):
     def __init__(self, addr, *args):
         ContainerTag.__init__(self, self.__class__.__name__, addr, args)
