@@ -13,7 +13,7 @@
  # implied.  See the License for the specific language governing
  # permissions and limitations under the License.
  #
- # $Id: test.py 280650 2005-09-13 20:35:57Z nlehuen $
+ # $Id: test.py 344174 2005-11-14 18:09:49Z nlehuen $
  #
 
 """
@@ -127,6 +127,9 @@ import time
 import socket
 import tempfile
 import base64
+import random
+import md5
+from cStringIO import StringIO
 
 HTTPD = testconf.HTTPD
 TESTHOME = testconf.TESTHOME
@@ -138,7 +141,16 @@ SERVER_ROOT = TESTHOME
 CONFIG = os.path.join(TESTHOME, "conf", "test.conf")
 DOCUMENT_ROOT = os.path.join(TESTHOME, "htdocs")
 PORT = 0 # this is set in fundUnusedPort()
-
+ 
+    
+# readBlockSize is required for the test_fileupload_* tests.
+# We can't import mod_python.util.readBlockSize from a cmd line
+# interpreter, so we'll hard code it here.
+# If util.readBlockSize changes, it MUST be changed here as well.
+# Maybe we should set up a separate test to query the server to 
+# get the correct readBlockSize?
+# 
+readBlockSize = 65368
 
 def findUnusedPort():
 
@@ -239,12 +251,12 @@ class HttpdCtrl:
         f.write("\n# --APPENDED-- \n\n"+append)
         f.close()
 
-    def startHttpd(self):
+    def startHttpd(self,extra=''):
 
         print "  Starting Apache...."
         httpd = quoteIfSpace(HTTPD)
         config = quoteIfSpace(CONFIG)
-        cmd = '%s -k start -f %s' % (httpd, config)
+        cmd = '%s %s -k start -f %s' % (httpd, extra, config)
         print "    ", cmd
         os.system(cmd)
         time.sleep(1)
@@ -259,6 +271,22 @@ class HttpdCtrl:
         print "    ", cmd
         os.system(cmd)
         time.sleep(1)
+
+        # Wait for apache to stop by checking for the existence of pid the 
+        # file. If pid file still exists after 20 seconds raise an error.
+        # This check is here to facilitate testing on the qemu emulator.
+        # Qemu will run about 1/10 the native speed, so 1 second may
+        # not be long enough for apache to shut down.
+        count = 0
+        pid_file = os.path.join(os.getcwd(), 'logs/httpd.pid')
+        while os.path.exists(pid_file):
+            time.sleep(1)
+            count += 1
+            if count > 20:
+                # give up - apache refuses to die - or died a horrible
+                # death and never removed the pid_file.
+                raise RuntimeError, "  Trouble stopping apache"
+
         self.httpd_running = 0
 
 class PerRequestTestCase(unittest.TestCase):
@@ -288,6 +316,68 @@ class PerRequestTestCase(unittest.TestCase):
 
         return rsp
 
+    def vhost_post_multipart_form_data(self, vhost, path="/tests.py",variables={}, files={}):
+        # variables is a { name : value } dict
+        # files is a { name : (filename, content) } dict
+
+        # build the POST entity
+        entity = StringIO()
+        # This is the MIME boundary
+        boundary = "============="+''.join( [ random.choice('0123456789') for x in range(10) ] )+'=='
+        # A part for each variable
+        for name, value in variables.iteritems():
+            entity.write('--')
+            entity.write(boundary)
+            entity.write('\r\n')
+            entity.write('Content-Type: text/plain\r\n')
+            entity.write('Content-Disposition: form-data; name="%s"\r\n'%name)
+            entity.write('\r\n')
+            entity.write(str(value))
+            entity.write('\r\n')
+
+        # A part for each file
+        for name, filespec in files.iteritems():
+            filename, content = filespec
+            # if content is readable, read it
+            try:
+                content = content.read()
+            except:
+                pass
+
+            entity.write('--')
+            entity.write(boundary)
+            entity.write('\r\n')
+
+            entity.write('Content-Type: application/octet-stream\r\n')
+            entity.write('Content-Disposition: form-data; name="%s"; filename="%s"\r\n'%(name,filename))
+            entity.write('\r\n')
+            entity.write(content)
+            entity.write('\r\n')
+
+        # The final boundary
+        entity.write('--')
+        entity.write(boundary)
+        entity.write('--\r\n')
+        
+        entity = entity.getvalue()
+
+        conn = httplib.HTTPConnection("127.0.0.1:%s" % PORT)
+        # conn.set_debuglevel(1000)
+        conn.putrequest("POST", path, skip_host=1)
+        conn.putheader("Host", "%s:%s" % (vhost, PORT))
+        conn.putheader("Content-Type", 'multipart/form-data; boundary="%s"'%boundary)
+        conn.putheader("Content-Length", '%s'%(len(entity)))
+        conn.endheaders()
+
+        start = time.time()
+        conn.send(entity)
+        response = conn.getresponse()
+        rsp = response.read()
+        conn.close()
+        print '    --> Send + process + receive took %.3f s'%(time.time()-start)
+
+        return rsp
+
     ### Tests begin here
 
     def test_req_document_root_conf(self):
@@ -307,7 +397,7 @@ class PerRequestTestCase(unittest.TestCase):
         print "\n  * Testing req.document_root()"
         rsp = self.vhost_get("test_req_document_root")
 
-        if rsp != DOCUMENT_ROOT.replace("\\", "/"):
+        if rsp.upper() != DOCUMENT_ROOT.replace("\\", "/").upper():
             self.fail(`rsp`)
 
     def test_req_add_handler_conf(self):
@@ -618,13 +708,232 @@ class PerRequestTestCase(unittest.TestCase):
 
     def test_req_sendfile(self):
 
-        print "\n  * Testing req.sendfile()"
+        print "\n  * Testing req.sendfile() with offset and length"
 
         rsp = self.vhost_get("test_req_sendfile")
 
         if (rsp != "test ok"):
             self.fail(`rsp`)
 
+    def test_req_sendfile2_conf(self):
+
+        c = VirtualHost("*",
+                        ServerName("test_req_sendfile2"),
+                        DocumentRoot(DOCUMENT_ROOT),
+                        Directory(DOCUMENT_ROOT,
+                                  SetHandler("mod_python"),
+                                  PythonHandler("tests::req_sendfile2"),
+                                  PythonDebug("On")))
+
+        return str(c)
+
+    def test_req_sendfile2(self):
+
+        print "\n  * Testing req.sendfile() without offset and length"
+
+        rsp = self.vhost_get("test_req_sendfile2")
+
+        if (rsp != "0123456789"*100):
+            self.fail(`rsp`)
+
+    def test_req_sendfile3_conf(self):
+
+        c = VirtualHost("*",
+                        ServerName("test_req_sendfile3"),
+                        DocumentRoot(DOCUMENT_ROOT),
+                        Directory(DOCUMENT_ROOT,
+                                  SetHandler("mod_python"),
+                                  PythonHandler("tests::req_sendfile3"),
+                                  PythonDebug("On")))
+
+        return str(c)
+
+    def test_req_sendfile3(self):
+
+        if os.name == 'posix':
+
+            print "\n  * Testing req.sendfile() for a file which is a symbolic link"
+
+            rsp = self.vhost_get("test_req_sendfile3")
+
+            if (rsp != "0123456789"*100):
+                self.fail(`rsp`)
+        else:
+            print "\n  * Skipping req.sendfile() for a file which is a symbolic link"
+
+    def test_fileupload_conf(self):
+
+        c = VirtualHost("*",
+                        ServerName("test_fileupload"),
+                        DocumentRoot(DOCUMENT_ROOT),
+                        Directory(DOCUMENT_ROOT,
+                                  SetHandler("mod_python"),
+                                  PythonHandler("tests::fileupload"),
+                                  PythonDebug("On")))
+
+        return str(c)
+
+    def test_fileupload(self):
+    
+        print "\n  * Testing 1 MB file upload support"
+
+        content = ''.join( [ chr(random.randrange(256)) for x in xrange(1024*1024) ] )
+        digest = md5.new(content).hexdigest()
+
+        rsp = self.vhost_post_multipart_form_data(
+            "test_fileupload",
+            variables={'test':'abcd'},
+            files={'testfile':('test.txt',content)},
+        )
+
+        if (rsp != digest):
+            self.fail('1 MB file upload failed, its contents were corrupted (%s)'%rsp)
+   
+    def test_fileupload_embedded_cr_conf(self):
+
+        c = VirtualHost("*",
+                        ServerName("test_fileupload"),
+                        DocumentRoot(DOCUMENT_ROOT),
+                        Directory(DOCUMENT_ROOT,
+                                  SetHandler("mod_python"),
+                                  PythonHandler("tests::fileupload"),
+                                  PythonDebug("On")))
+
+        return str(c)
+
+    def test_fileupload_embedded_cr(self):
+        # Strange things can happen if there is a '\r' character at position
+        # readBlockSize of a line being read by FieldStorage.read_to_boundary 
+        # where the line length is > readBlockSize.
+        # This test will expose this problem.
+        
+        print "\n  * Testing file upload with \\r char in a line at position == readBlockSize"
+        
+        content = (
+            'a'*100 + '\r\n'
+            + 'b'*(readBlockSize-1) + '\r' # trick !
+            + 'ccc' + 'd'*100 + '\r\n'
+        )
+        digest = md5.new(content).hexdigest()
+        
+        rsp = self.vhost_post_multipart_form_data(
+            "test_fileupload",
+            variables={'test':'abcd'},
+            files={'testfile':('test.txt',content)},
+        )
+
+        if (rsp != digest):
+            self.fail('file upload embedded \\r test failed, its contents were corrupted (%s)'%rsp)
+
+        # The UNIX-HATERS handbook illustrates this problem. Once we've done some additional
+        # investigation to make sure that our synthetic file used above is correct,
+        # we can likely remove this conditional test. Also, there is no way to be sure
+        # that ugh.pdf will always be the same in the future so the test may not be valid
+        # over the long term.
+        try:
+            ugh = file('ugh.pdf','rb')
+            content = ugh.read()
+            ugh.close()
+        except:
+            print "  * Skipping the test for The UNIX-HATERS handbook file upload."
+            print "    To make this test, you need to download ugh.pdf from"
+            print "    http://research.microsoft.com/~daniel/uhh-download.html"
+            print "    into this script's directory."
+        else:
+            print "  * Testing The UNIX-HATERS handbook file upload support"
+    
+            digest = md5.new(content).hexdigest()
+    
+            rsp = self.vhost_post_multipart_form_data(
+                "test_fileupload",
+                variables={'test':'abcd'},
+                files={'testfile':('ugh.pdf',content)},
+            )
+    
+            
+            if (rsp != digest):
+                self.fail('The UNIX-HATERS handbook file upload failed, its contents was corrupted (%s)'%rsp)
+
+    def test_fileupload_split_boundary_conf(self):
+
+        c = VirtualHost("*",
+                        ServerName("test_fileupload"),
+                        DocumentRoot(DOCUMENT_ROOT),
+                        Directory(DOCUMENT_ROOT,
+                                  SetHandler("mod_python"),
+                                  PythonHandler("tests::fileupload"),
+                                  PythonDebug("On")))
+
+        return str(c)
+
+    def test_fileupload_split_boundary(self):
+        # This test is similar to test_fileupload_embedded_cr, but it is possible to
+        # write an implementation of FieldStorage.read_to_boundary that will pass
+        # that test but fail this one.
+        # 
+        # Strange things can happen if the last line in the file being uploaded 
+        # has length == readBlockSize -1. The boundary string marking the end of the
+        # file (eg. '\r\n--myboundary') will be split between the leading '\r' and the
+        # '\n'. Some implementations of read_to_boundary we've tried assume that this
+        # '\r' character is part of the file, instead of the boundary string. The '\r'
+        # will be appended to the uploaded file, leading to a corrupted file.
+
+        print "\n  * Testing file upload where length of last line == readBlockSize - 1"
+
+        content = (
+            'a'*100 + '\r\n'
+            + 'b'*(readBlockSize-1)  # trick !
+        )
+        digest = md5.new(content).hexdigest()
+        
+        rsp = self.vhost_post_multipart_form_data(
+            "test_fileupload",
+            variables={'test':'abcd'},
+            files={'testfile':('test.txt',content)},
+        )
+
+        if (rsp != digest):
+            self.fail('file upload long line test failed, its contents were corrupted (%s)'%rsp)
+
+        print "  * Testing file upload where length of last line == readBlockSize - 1 with an extra \\r"
+
+        content = (
+            'a'*100 + '\r\n'
+            + 'b'*(readBlockSize-1)
+            + '\r'  # second trick !
+        )
+        digest = md5.new(content).hexdigest()
+        
+        rsp = self.vhost_post_multipart_form_data(
+            "test_fileupload",
+            variables={'test':'abcd'},
+            files={'testfile':('test.txt',content)},
+        )
+
+        if (rsp != digest):
+            self.fail('file upload long line test failed, its contents were corrupted (%s)'%rsp)
+
+    def test_sys_argv_conf(self):
+
+        c = VirtualHost("*",
+                        ServerName("test_sys_argv"),
+                        DocumentRoot(DOCUMENT_ROOT),
+                        Directory(DOCUMENT_ROOT,
+                                  SetHandler("mod_python"),
+                                  PythonHandler("tests::test_sys_argv"),
+                                  PythonDebug("On")))
+
+        return str(c)
+
+    def test_sys_argv(self):
+
+        print "\n  * Testing sys.argv definition"
+
+        rsp = self.vhost_get("test_sys_argv")
+
+        if (rsp != "['mod_python']"):
+            self.fail(`rsp`)
+
     def test_PythonOption_conf(self):
 
         c = VirtualHost("*",
@@ -733,17 +1042,17 @@ class PerRequestTestCase(unittest.TestCase):
 
         print "\n  * Testing interpreter per directive"
 
-        interpreter_name = DOCUMENT_ROOT.replace('\\', '/')+'/'
+        interpreter_name = (DOCUMENT_ROOT.replace('\\', '/')+'/').upper()
 
-        rsp = self.vhost_get("test_interpreter_per_directive")
+        rsp = self.vhost_get("test_interpreter_per_directive").upper()
         if (rsp != interpreter_name):
             self.fail(`rsp`)
 
-        rsp = self.vhost_get("test_interpreter_per_directive", '/subdir/foo.py')
+        rsp = self.vhost_get("test_interpreter_per_directive", '/subdir/foo.py').upper()
         if (rsp != interpreter_name):
             self.fail(`rsp`)
 
-        rsp = self.vhost_get("test_interpreter_per_directive", '/subdir/')
+        rsp = self.vhost_get("test_interpreter_per_directive", '/subdir/').upper()
         if (rsp != interpreter_name):
             self.fail(`rsp`)
 
@@ -765,18 +1074,18 @@ class PerRequestTestCase(unittest.TestCase):
 
         print "\n  * Testing interpreter per directory"
 
-        interpreter_name = DOCUMENT_ROOT.replace('\\', '/')+'/'
+        interpreter_name = (DOCUMENT_ROOT.replace('\\', '/')+'/').upper()
 
-        rsp = self.vhost_get("test_interpreter_per_directory")
+        rsp = self.vhost_get("test_interpreter_per_directory").upper()
         if (rsp != interpreter_name):
             self.fail(`rsp`)
 
-        rsp = self.vhost_get("test_interpreter_per_directory", '/subdir/foo.py')
-        if (rsp != interpreter_name+'subdir/'):
+        rsp = self.vhost_get("test_interpreter_per_directory", '/subdir/foo.py').upper()
+        if (rsp != interpreter_name+'SUBDIR/'):
             self.fail(`rsp`)
 
-        rsp = self.vhost_get("test_interpreter_per_directory", '/subdir/')
-        if (rsp != interpreter_name+'subdir/'):
+        rsp = self.vhost_get("test_interpreter_per_directory", '/subdir/').upper()
+        if (rsp != interpreter_name+'SUBDIR/'):
             self.fail(`rsp`)
 
     def test_util_fieldstorage_conf(self):
@@ -926,6 +1235,7 @@ class PerRequestTestCase(unittest.TestCase):
                         Directory(DOCUMENT_ROOT,
                                   SetHandler("mod_python"),
                                   PythonHandler("tests"),
+                                  PythonOption('PythonOptionTest ""'),
                                   PythonOption("testing 123"),
                                   PythonDebug("On")))
         return str(c)
@@ -1001,6 +1311,54 @@ class PerRequestTestCase(unittest.TestCase):
         if (rsp[-8:] != "test ok\n"):
             self.fail(`rsp`)
 
+    def test_psp_parser_conf(self):
+
+        c = VirtualHost("*",
+                        ServerName("test_psp_parser"),
+                        DocumentRoot(DOCUMENT_ROOT),
+                        Directory(DOCUMENT_ROOT,
+                                  SetHandler("mod_python"),
+                                  PythonHandler("mod_python.psp"),
+                                  PythonDebug("On")))
+        return str(c)
+
+    def test_psp_parser(self):
+
+        print "\n  * Testing mod_python.psp parser"
+                # lines in psp_parser.psp should look like:
+        #   test:<char>:<test_string>$
+        #
+        # For example:
+        #   test:n:\n$
+        #   test:t:\t$
+        
+        rsp = self.vhost_get("test_psp_parser", path="/psp_parser.psp")
+        lines = [ line.strip() for line in rsp.split('$') if line ]
+        failures = []
+        for line in lines:
+            parts =  line.split(':', 2)
+            if len(parts) < 3:
+                continue
+
+            t, test_case, test_string = parts[0:3]
+            if not t.strip().startswith('test'):
+                continue
+            expected_result = test_case
+            # do the substitutions in expected_result
+            for ss, rs in [('-', '\\'),('CR', '\r'), ('LF', '\n'), ('TB', '\t')]:
+                expected_result = expected_result.replace(ss, rs)
+            
+            if expected_result != test_string:
+                print '       FAIL',  
+                failures.append(test_case)
+            else:
+                print '       PASS', 
+            print 'expect{%s} got{%s}' % (expected_result, test_string)
+
+        if failures:
+            msg = 'psp_parser parse errors for: %s' % (', '.join(failures))
+            self.fail(msg)
+
     def test_Cookie_Cookie_conf(self):
 
         c = VirtualHost("*",
@@ -1350,6 +1708,7 @@ class PerInstanceTestCase(unittest.TestCase, HttpdCtrl):
         self.failUnless(server_hdr.find("Python") > -1,
                         "%s does not appear to load, Server header does not contain Python"
                         % MOD_PYTHON_SO)
+
     def test_global_lock(self):
 
         print "\n  * Testing _global_lock"
@@ -1375,7 +1734,7 @@ class PerInstanceTestCase(unittest.TestCase, HttpdCtrl):
         print "    ", time.ctime()
         ab = quoteIfSpace(AB)
         if os.name == "nt":
-            cmd = '%s -c 5 -n 5 http://127.0.0.1:%s/tests.py > null' \
+            cmd = '%s -c 5 -n 5 http://127.0.0.1:%s/tests.py > NUL:' \
                   % (ab, PORT)
         else:
             cmd = '%s -c 5 -n 5 http://127.0.0.1:%s/tests.py > /dev/null' \
@@ -1386,7 +1745,7 @@ class PerInstanceTestCase(unittest.TestCase, HttpdCtrl):
         t2 = time.time()
         if (t2 - t1) < 5:
             self.fail("global_lock is broken (too quick)")
-         
+
     def testPerRequestTests(self):
 
         print "\n* Running the per-request test suite..."
@@ -1404,7 +1763,12 @@ class PerInstanceTestCase(unittest.TestCase, HttpdCtrl):
         perRequestSuite.addTest(PerRequestTestCase("test_req_register_cleanup"))
         perRequestSuite.addTest(PerRequestTestCase("test_req_headers_out"))
         perRequestSuite.addTest(PerRequestTestCase("test_req_sendfile"))
-        perRequestSuite.addTest(PerRequestTestCase("test_PythonOption"))
+        perRequestSuite.addTest(PerRequestTestCase("test_req_sendfile2"))
+        perRequestSuite.addTest(PerRequestTestCase("test_req_sendfile3"))
+        perRequestSuite.addTest(PerRequestTestCase("test_fileupload"))
+        perRequestSuite.addTest(PerRequestTestCase("test_fileupload_embedded_cr"))
+        perRequestSuite.addTest(PerRequestTestCase("test_fileupload_split_boundary"))
+        perRequestSuite.addTest(PerRequestTestCase("test_sys_argv"))
         perRequestSuite.addTest(PerRequestTestCase("test_PythonOption_override"))
         perRequestSuite.addTest(PerRequestTestCase("test_PythonOption_remove"))
         perRequestSuite.addTest(PerRequestTestCase("test_PythonOption_remove2"))
@@ -1417,6 +1781,7 @@ class PerInstanceTestCase(unittest.TestCase, HttpdCtrl):
         perRequestSuite.addTest(PerRequestTestCase("test_pipe_ext"))
         perRequestSuite.addTest(PerRequestTestCase("test_cgihandler"))
         perRequestSuite.addTest(PerRequestTestCase("test_psphandler"))
+        perRequestSuite.addTest(PerRequestTestCase("test_psp_parser"))
         perRequestSuite.addTest(PerRequestTestCase("test_Cookie_Cookie"))
         perRequestSuite.addTest(PerRequestTestCase("test_Cookie_MarshalCookie"))
         perRequestSuite.addTest(PerRequestTestCase("test_Session_Session"))
@@ -1429,7 +1794,7 @@ class PerInstanceTestCase(unittest.TestCase, HttpdCtrl):
         perRequestSuite.addTest(PerRequestTestCase("test_publisher_iterator"))
         perRequestSuite.addTest(PerRequestTestCase("test_publisher_hierarchy"))
         # this must be last so its error_log is not overwritten
-        # perRequestSuite.addTest(PerRequestTestCase("test_internal"))
+        perRequestSuite.addTest(PerRequestTestCase("test_internal"))
 
         self.makeConfig(PerRequestTestCase.appendConfig)
         self.startHttpd()
@@ -1497,14 +1862,47 @@ class PerInstanceTestCase(unittest.TestCase, HttpdCtrl):
         if log.find("test 2 ok") == -1:
             self.fail("Could not find test message in error_log")
 
+    def test_apache_exists_config_define(self):
+
+        print "\n* Testing apache.exists_config_define()..."
+
+        c = Directory(DOCUMENT_ROOT,
+                      SetHandler("mod_python"),
+                      PythonHandler("tests::apache_exists_config_define"),
+                      PythonDebug("On"))
+
+        self.makeConfig(str(c))
+
+        self.startHttpd()
+
+        f = urllib.urlopen("http://127.0.0.1:%s/tests.py" % PORT)
+        rsp = f.read()
+        f.close()
+
+        self.stopHttpd()
+        
+        if rsp != 'NO_FOOBAR':
+            self.fail('Failure on apache.exists_config_define() : %s'%rsp)
+
+        self.startHttpd(extra="-DFOOBAR")
+
+        f = urllib.urlopen("http://127.0.0.1:%s/tests.py" % PORT)
+        rsp = f.read()
+        f.close()
+
+        self.stopHttpd()
+        
+        if rsp != 'FOOBAR':
+            self.fail('Failure on apache.exists_config_define() : %s'%rsp)
+
 def suite():
 
     mpTestSuite = unittest.TestSuite()
     mpTestSuite.addTest(PerInstanceTestCase("testLoadModule"))
     mpTestSuite.addTest(PerInstanceTestCase("test_srv_register_cleanup"))
     mpTestSuite.addTest(PerInstanceTestCase("test_apache_register_cleanup"))
-    # XXX comment out until ab is fixed, or we have another way
-##     mpTestSuite.addTest(PerInstanceTestCase("test_global_lock"))
+    mpTestSuite.addTest(PerInstanceTestCase("test_apache_exists_config_define"))
+    mpTestSuite.addTest(PerInstanceTestCase("test_global_lock"))
     mpTestSuite.addTest(PerInstanceTestCase("testPerRequestTests"))
     return mpTestSuite
 
